// Jean Luc Biellmann - contact@alsatux.com

"use strict";

var _EventDB = {
	seq: 0,
	db: {},
	add: function () {
		var seq = 'seq_' + _EventDB.seq++;
		var now = new Date();
		return _EventDB.db[seq] = {
			'seq': seq,
			'start_date': now.toYYYYMMDD('-'),
			'start_hour': now.getHours(),
			'start_minute': now.getMinutes(),
			'duration_hour': 1,
			'duration_minute': 0,
			'title': '',
			'description': '',
			'rrule': {
				'freq': '',
				'count': 0,
				'interval': 1,
				'until': ''
			}
		};
	},
	del: function (seq) {
		delete _EventDB.db[seq];
	},
	upd: function (obj) {
		if (!('seq' in obj)) {
			_Log.error('No sequence given.');
			return false;
		}
		var seq = obj['seq'];
		if (!(seq in _EventDB.db)) {
			_Log.error('Unknow sequence.');
			return false;
		}
		if ('start_date' in obj) {
			var start_date = obj['start_date'];
			if (!_Check.is_valid_date(start_date)) {
				_Log.error('Invalid start date.');
				return false;
			}
		}
		if ('start_hour' in obj)
			obj['start_hour'] = _Fmt.modulo(obj['start_hour'], 24);
		if ('start_minute' in obj)
			obj['start_minute'] = _Fmt.modulo(obj['start_minute'], 60);
		if ('duration_hour' in obj)
			obj['duration_hour'] = _Fmt.modulo(obj['duration_hour'], 24*365);
		if ('duration_minute' in obj)
			obj['duration_minute'] = _Fmt.modulo(obj['duration_minute'], 60);
		if ('rrule' in obj) {
			if ('freq' in obj['rrule'] && obj['rrule']['freq'].match(/^(|yearly|monthly|weekly|daily)$/)==null) {
				_Log.error('Invalid rrule freq.');
				return false;
			}
			if ('count' in obj['rrule'])
				obj['rrule']['count'] = _Fmt.modulo(obj['rrule']['count'], 99);
			if ('interval' in obj['rrule'])
				obj['rrule']['interval'] = _Fmt.modulo(obj['rrule']['interval'], 99);
			if ('until' in obj['rrule']) {
				var until = obj['rrule']['until'];
				if (until.length && !_Check.is_valid_date(until)) {
					_Log.error('Invalid until date.');
					return false;
				}
			}
		}
		for (var key in obj) {
			if (key=='rrule') {
				for (var key2 in obj['rrule'])
					_EventDB.db[seq]['rrule'][key2] = obj['rrule'][key2];
			} else {
				//_Log.info('_EventDB.db[' + seq + '][' + key + '] = ' + obj[key]);
				_EventDB.db[seq][key] = obj[key];
			}
		}
		return true;
	},
	save: function () {
		var now = new Date();
		var yyyy = now.getFullYear();
		var mm = _Fmt.two_digits(now.getMonth() + 1);
		var dd = _Fmt.two_digits(now.getDate());
		var hh = _Fmt.two_digits(now.getHours());
		var ii = _Fmt.two_digits(now.getMinutes());
		var ss = _Fmt.two_digits(now.getSeconds());
		_XML.write('cactus/default/' + yyyy + mm + dd + hh + ii + ss + '.xml')
	}
};

